import os
import chainlit as cl
from openai import AsyncOpenAI, OpenAI
from chainlit.config import config

from test_handler import EventHandler

async_openai_client = AsyncOpenAI(api_key=os.environ.get("OPENAI_API_KEY"))
sync_openai_client = OpenAI(api_key=os.environ.get("OPENAI_API_KEY"))

cl.instrument_openai()

assistant = sync_openai_client.beta.assistants.retrieve(
    os.environ.get("OPENAI_ASSISTANT_ID")
)

config.ui.name = assistant.name

@cl.on_chat_start
async def start_chat():
    thread = await async_openai_client.beta.threads.create()

    cl.user_session.set("thread_id", thread.id)

    await cl.Message(content=f"Hello world, I'm {assistant.name}! How can I assist you?").send()

@cl.on_message
async def on_message(message: cl.Message):
    thread_id = cl.user_session.get("thread_id")

    # Add a Message to the Thread
    oai_message = await async_openai_client.beta.threads.messages.create(
        thread_id=thread_id,
        role="user",
        content=message.content,
        attachments=[],
    )

    # Create and Stream a Run
    async with async_openai_client.beta.threads.runs.stream(
        thread_id=thread_id,
        assistant_id=assistant.id,
        event_handler=EventHandler(
            assistant_name=assistant.name, 
            cl=cl, 
            client=async_openai_client
        ),
    ) as stream:
        await stream.until_done()